#pragma once

/*
MIT License
Copyright (c) 2024 - 2025 René Amthor (tobid7)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */

#include <pd/core/core.hpp>
#include <pd/lithium/renderer.hpp>
#include <pd/overlays/overlay.hpp>

namespace PD {
/**
 * Overlay Manager Class
 */
class OverlayMgr : public SmartCtor<OverlayMgr> {
 public:
  /**
   * Constructor
   * @param ren Renderer
   * @param inp Input Driver reference
   */
  OverlayMgr(LI::Renderer::Ref ren, Hid::Ref inp) {
    this->ren = ren;
    this->inp = inp;
  }
  /** Deconstructor */
  ~OverlayMgr() { overlays.clear(); }

  /**
   * Append a New Overlay
   * @param overlay Overlay reference to push
   */
  void Push(Overlay::Ref overlay);
  /**
   * Update Overlays
   * @paran delta Deltatime
   */
  void Update(float delta);

 private:
  std::vector<Overlay::Ref> overlays;  ///< Overlay List
  LI::Renderer::Ref ren;               ///< Renderer reference
  Hid::Ref inp;                        ///< Input Driver reference
};
}  // namespace PD